//Written by Andy Friesen
import c.stdio;
import stream;
import formatter;

private class ConsoleStream : Stream
{
public:
    this()
    {
        readable = writeable = true;
    }

    override uint readBlock(void* buffer, uint size)
    {
        byte* b = cast(byte*)buffer;
        uint i = size;
        while (i--)
            *(b++) = c.stdio.getchar();
        return size;
    }

    override uint writeBlock(void* buffer, uint size)
    {
        uint r= c.stdio.fwrite(buffer, 1, size, c.stdio.stdout);
	fflush(c.stdio.stdout);
	return r;
    }

    override ulong seek(long ofs, SeekPos whence)
    {
        return 0;
    }

    override void read(out char ch)
    {
        ch = c.stdio.getchar();
    }

    // Needed to be unicode-friendly.  I get a link error on getwchar, though.
    /*override void read(out wchar ch)
    {
        ch = c.stdio.getwchar();
    }*/

    void write(Formatter fmt)
    {
        writeString(fmt.toString());
    }

	void write(char[] str) { writeString(str); }

	void write (char ch)
	{
		char[] s; s~=ch; writeString(s);
	}

	void write (Object o) { write (o.toString()); }
}

class Console
{
	//Modified by Carlos Santander B.
	import disp;
	
	public static ConsoleStream io;
    
	static this()
    {
        io = new ConsoleStream();
		disp_open();
		int attr=disp_getattr();
		_fg=attr % 16;
		_bg=attr / 16;
    }
	
	static ~this() { disp_close(); }
	
	static void clrscr() {
		disp_move(0,0);
		disp_eeop();
	}
	
	static void gotoxy(int x,int y) {
		disp_move(y,x);
	}
	
	static void set43rows() { disp_set43(); }
	static void set25rows() { disp_reset43(); }
	
	static void showcursor(bit show) { //doesn't work properly
		if (show) disp_showcursor();
		else disp_hidecursor();
	}
	
	private static int _fg,_bg;
	
	static void textcolor(int fg) { color(fg,_bg); }
	
	static void textbackground(int bg) { color(_fg,bg); }
	
	static void color(int foreground) { color(foreground,_bg); }
	
	static void color(int foreground,int background) {
		disp_setattr(background*16+foreground);
		_fg=foreground;
		_bg=background;
	}
	
	static void getcolor(out int foreground,out int background) {
		foreground=_fg;
		background=_bg;
	}
	
	static void refresh() { disp_flush(); }
	
	/* From now on:
	 * 	trow: top row
	 * 	brow: bottom row
	 * 	lcol: left column
	 * 	rcol: right column	
	 */
	static void box(uint trow, uint lcol, uint brow, uint rcol,int type) {
		disp_box(type,_bg*16+_fg,trow,lcol,brow,rcol);
	}
	
	static void hline(uint row,uint lcol,uint rcol,int type)  {
		disp_move(row,lcol);
		char c;
		switch (type) {
			case 0: c=205; break;
			case 2: c=219; break;
			default: c=196; break;
		}
		for (int i=lcol;i<=rcol;i++)
			disp_putc(c);
	}
	
	static void vline(uint col,uint trow,uint brow,int type) {
		char c;
		switch (type) {
			case 0: c=186; break;
			case 2: c=219; break;
			default: c=179; break;
		}
		for (int i=trow;i<=brow;i++) {
			disp_move(i,col);
			disp_putc(c);
		}
	}
	
	static void fill(uint trow, uint lcol, uint brow, uint rcol) { //doesn't work properly
		disp_fillbox(_bg*16+_fg,trow,lcol,brow,rcol);
	}
	
	//given row,col, assigns to c the char, to fg the color and to bg the background color at that position
	static void peek(int row,int col,out char c,out int fg,out int bg) {
		ushort attr=disp_peekw(row,col);
		c=attr%256;
		attr/=256;
		fg=attr%16;
		bg=attr/16;
	}
	
	//puts the char c at row,col, using fg as color and bg as background color
	static void poke(int row,int col,char c,int fg,int bg) {
		ushort attr=bg*16+fg;
		disp_pokew(row,col,attr*256+c);
	}
	
	//given lines, scrolls the rectangle defined by (trow,lcol),(brow,rcol), and fills the uncovered areas with the current background color
	//if lines>0, scrolls up. if lines<0, scrolls down
	static void scroll(int lines, uint trow, uint lcol, uint brow, uint rcol) {
		disp_scroll(lines,trow,lcol,brow,rcol,0);
	}
}

Formatter format(char[] str)
{
    // I see no way to do this without creating an object.
    // Using a single static instance raises threading issues
    return new Formatter(str);
}

unittest
{
    char[] blah = "BOOOOOOOOOOOOOOOOOOO";
    Console.io.writeString("blah!");
    Console.io.write(format("String: \"{%}\" {%} + {%} = {%}") % blah % 5 % 3 % (5+3));
}

version (UNITTEST)
void main(){}